/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original author
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "collidingSolidParticleCloud.H"

// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

bool Foam::collidingSolidParticle::move(collidingSolidParticle::trackData& td)
{
    td.switchProcessor = false;
    td.keepParticle = true;

    const polyMesh& mesh = cloud().pMesh();
    const polyBoundaryMesh& pbMesh = mesh.boundaryMesh();

    scalar deltaT = mesh.time().deltaT().value();
    scalar tEnd = (1.0 - stepFraction())*deltaT;
    scalar dtMax = tEnd;

    while (td.keepParticle && !td.switchProcessor && tEnd > SMALL)
    {
        if (debug)
        {
            Info<< "Time = " << mesh.time().timeName()
                << " deltaT = " << deltaT
                << " tEnd = " << tEnd
                << " steptFraction() = " << stepFraction() << endl;
        }

        // set the lagrangian time-step
        scalar dt = min(dtMax, tEnd);

        // remember which cell the parcel is in
        // since this will change if a face is hit
        label celli = cell();

        dt *= trackToFace(position() + dt*U_, td);

        tEnd -= dt;
        stepFraction() = 1.0 - tEnd/deltaT;

        cellPointWeight cpw(mesh, position(), celli, face());
        scalar rhoc = td.rhoInterp().interpolate(cpw);
        vector Uc = td.UInterp().interpolate(cpw);
        scalar nuc = td.nuInterp().interpolate(cpw);

        scalar rhop = td.spc().rhop();
        scalar magUr = mag(Uc - U_);

        scalar ReFunc = 1.0;
        scalar Re = magUr*d_/nuc;

        if (Re > 0.01)
        {
            ReFunc += 0.15*pow(Re, 0.687);
        }

        scalar Dc = (24.0*nuc/d_)*ReFunc*(3.0/4.0)*(rhoc/(d_*rhop));

        scalar e = td.spc().e(); //Restitution coefficient
scalar m = rhop*d_*d_*d_*mathematicalConstant::pi*4.0/3.0; //Mass
bool checkcoll=td.spc().collision();
if(checkcoll){Info<<"Particles collide!"<<endl;}
vector V0=td.spc().U0(); //OLD velocity of particle 1
scalar Jx = -(1.0+e)*V0.x()*m*m/(2.0*m);//Impulsive force x-comp
scalar Jy = -2.0/7.0*V0.y()*m*m/(2.0*m);//Impulsive force y-comp
if(ID_==0 && checkcoll) //Collision for particle 1
{
U_.x()=V0.x()+Jx/m;
U_.y()=V0.y()+Jy/m;
U_.z()=0.0;
}
if(ID_==1 && checkcoll) //Collision for particle 2
{
U_.x() = -Jx/m;
U_.y() = -Jy/m;
U_.z() = 0.0;
}
//If no collision
U_ = (U_ + dt*(Dc*Uc + (1.0 - rhoc/rhop)*td.g()))/(1.0 + dt*Dc);

        if (onBoundary() && td.keepParticle)
        {
            if (face() > -1)
            {
                if (isType<processorPolyPatch>(pbMesh[patch(face())]))
                {
                    td.switchProcessor = true;
                }
            }
        }
    }

    return td.keepParticle;
}


void Foam::collidingSolidParticle::hitProcessorPatch
(
    const processorPolyPatch&,
    collidingSolidParticle::trackData& td
)
{
    td.switchProcessor = true;
}


void Foam::collidingSolidParticle::hitProcessorPatch
(
    const processorPolyPatch&,
    int&
)
{}


void Foam::collidingSolidParticle::hitWallPatch
(
    const wallPolyPatch& wpp,
    collidingSolidParticle::trackData& td
)
{
    vector nw = wpp.faceAreas()[wpp.whichFace(face())];
    nw /= mag(nw);

    scalar Un = U_ & nw;
    vector Ut = U_ - Un*nw;

    if (Un > 0)
    {
        U_ -= (1.0 + td.spc().e())*Un*nw;
    }

    U_ -= td.spc().mu()*Ut;
}


void Foam::collidingSolidParticle::hitWallPatch
(
    const wallPolyPatch&,
    int&
)
{}


void Foam::collidingSolidParticle::hitPatch
(
    const polyPatch&,
    collidingSolidParticle::trackData& td
)
{
    td.keepParticle = false;
}


void Foam::collidingSolidParticle::hitPatch
(
    const polyPatch&,
    int&
)
{}


void Foam::collidingSolidParticle::transformProperties(const tensor& T)
{
    Particle<collidingSolidParticle>::transformProperties(T);
    U_ = transform(T, U_);
}


void Foam::collidingSolidParticle::transformProperties(const vector& separation)
{
    Particle<collidingSolidParticle>::transformProperties(separation);
}


// ************************************************************************* //
